﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("ThinPlatePolysplineKernel", "A kernel function that uses the ThinPlatePolyspline function")]
  public class ThinPlatePolysplineKernel : KernelBase {
    #region HLConstructors & Boilerplate
    [StorableConstructor]
    protected ThinPlatePolysplineKernel(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }
    protected ThinPlatePolysplineKernel(ThinPlatePolysplineKernel original, Cloner cloner) : base(original, cloner) { }
    public ThinPlatePolysplineKernel() {
      Parameters.Add(new FixedValueParameter<DoubleValue>(BetaParameterName, "The Beta in the kernelfunction ||x-c||^(2*Beta)*log(||x-c||^Beta)", new DoubleValue(1)));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ThinPlatePolysplineKernel(this, cloner);
    }
    #endregion

    protected override double Get(double norm) {
      if (Math.Pow(norm, Beta) <= 0) return 0;
      return Math.Pow(norm, Beta * 2) * Math.Log(1 + Math.Pow(norm, Beta));
    }

    protected override double GetGradient(double norm) {
      if (Math.Pow(norm, Beta) <= 0) return double.NaN;
      var ax = Math.Pow(norm, Beta);
      var a2x = ax * ax;
      var a3x = a2x * ax;
      var loga = Math.Log(norm);
      return 2 * a2x * loga * Math.Log(ax + 1) + a3x * loga / (ax + 1);
    }
  }
}
